/**
  ******************************************************************************
  * @file    usbd_audio.h
  * @author  MK
  * @version V0.2
  * @date    08/01/2021
  * @brief   Header file for common functions for USB Audio class 1&2.
  ******************************************************************************
  */
 
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __USB_AUDIO_H
#define __USB_AUDIO_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "config.h"
#include "usbd_ioreq.h"
#include <stdbool.h>

#ifndef USBD_AUDIO_FREQ_DEFAULT
#define USBD_AUDIO_FREQ_DEFAULT                       44100U
#endif

#ifndef USBD_AUDIO_FREQ_MAX
#define USBD_AUDIO_FREQ_MAX                           192000U
#endif

#ifndef USBD_AUDIO_NUM_CHANNELS
#define USBD_AUDIO_NUM_CHANNELS                       2U
#endif

#ifndef USBD_AUDIO_VOL_DEFAULT
#define USBD_AUDIO_VOL_DEFAULT                        0x8d00U
#endif

#ifndef USBD_AUDIO_VOL_MAX
#define USBD_AUDIO_VOL_MAX                            0x0000U
#endif

#ifndef USBD_AUDIO_VOL_MIN
#define USBD_AUDIO_VOL_MIN                            0x8100U
#endif

#ifndef USBD_AUDIO_VOL_STEP
#define USBD_AUDIO_VOL_STEP                           0x0100U
#endif

#ifndef USBD_MAX_NUM_INTERFACES
#define USBD_MAX_NUM_INTERFACES                       2U	// 1U
#endif /* USBD_MAX_NUM_INTERFACES */

#ifndef AUDIO_HS_BINTERVAL
#define AUDIO_HS_BINTERVAL                            0x01U
#endif /* AUDIO_HS_BINTERVAL */

#ifndef AUDIO_FS_BINTERVAL
#define AUDIO_FS_BINTERVAL                            0x01U
#endif /* AUDIO_FS_BINTERVAL */

#define AUDIO_OUT_EP                                  0x01
#define AUDIO_OUT_FB_EP                    			  0x81  // feedback endpoint
#define AUDIO_IN_EP                     			  0x82

#define AUDIO_OUT_PACKET(freq)      (uint16_t)((freq / 1000U) * 2U * 2U)

#define AUDIO_PACKET_SIZE(frq)      (uint8_t)(((frq/1000)+1) * 2 * 2) & 0xFF), \
                                    (uint8_t)(((frq/1000)+1) * 2 * 2) >> 8) & 0xFF)

#ifdef USE_USB_FS
#define MAX_PACKET_SIZE				((192 + 1) * 2 * 2)
#define AUDIO_OUT_BUF_SIZE 			128			// Audio buffer size for each channel, in samples
#define AUDIO_OUT_NUM_BUFFERS		8			// Even multiple
#define AUDIO_OUT_TOTAL_BUF_SIZE 	(AUDIO_OUT_BUF_SIZE * AUDIO_OUT_NUM_BUFFERS)
#define AUDIO_IN_SAMPLE_BUF_SIZE	256			// Samples
#else
#define MAX_PACKET_SIZE				1023
#define AUDIO_IN_PACKET_SIZE(freq,bytes_per_sample)      ((freq / 8000U) * 2U * bytes_per_sample)
#define AUDIO_IN_MAX_PACKET_SIZE    (((768 / 8) + 1) * 2 * 4)
#define AUDIO_IN_SAMPLE_BUF_SIZE	1024	// 64 256		// Record buffer size for each channel, in samples
#define AUDIO_IN_NUM_BUFFERS		8	// 8		// Even multiple
#define AUDIO_IN_TOTAL_BUF_SIZE		(AUDIO_IN_SAMPLE_BUF_SIZE * AUDIO_IN_NUM_BUFFERS)
#endif

#define IN_SAMPLE_MASK				(uint32_t)0xffffffff; //0xfffc0fff

#define IDLE_TIMEOUT				1000

// FB rate calculation
#define OUT_GAP_U2	(AUDIO_OUT_TOTAL_BUF_SIZE * 6 / 8)
#define	OUT_GAP_U1	(AUDIO_OUT_TOTAL_BUF_SIZE * 5 / 8)
#define OUT_GAP_NOM	(AUDIO_OUT_TOTAL_BUF_SIZE * 4 / 8)
#define OUT_GAP_L1	(AUDIO_OUT_TOTAL_BUF_SIZE * 3 / 8)
#define OUT_GAP_L2	(AUDIO_OUT_TOTAL_BUF_SIZE * 2 / 8)

#define IN_GAP_NOM	(AUDIO_IN_SAMPLE_BUF_SIZE * 4)
#define IN_GAP_HIGH (AUDIO_IN_SAMPLE_BUF_SIZE * 5)
#define IN_GAP_LOW  (AUDIO_IN_SAMPLE_BUF_SIZE * 3)


#define USBD_AUDIO_MAX_IN_EP        2
#define USBD_AUDIO_MAX_OUT_EP       1
#define USBD_AUDIO_MAX_AS_INTERFACE 2

#define PCM_SILENCE		0L
#define DSD_SILENCE		0x69696969L
#define RTZ_SILENCE		0x55555555L

 typedef enum
{
	IDLE = 1,
	STARTED,
	STREAMING,
	STOPPED,
} AUDIO_StateTypeDef;

typedef enum
{
	PARITY_EVEN    = 0,
	PARITY_ODD     = 1,
	PARITY_FLIP    = 2,
	PARITY_UNDEF   = 3
} AUDIO_ParityTypeDef;

/* Audio Commands enumeration */
typedef enum
{
  AUDIO_CMD_START = 1,
  AUDIO_CMD_PLAY,
  AUDIO_CMD_STOP
} AUDIO_CMD_TypeDef;

typedef enum
{
  AUDIO_OFFSET_NONE = 0,
  AUDIO_OFFSET_HALF,
  AUDIO_OFFSET_FULL,  
  AUDIO_OFFSET_UNKNOWN,    
} AUDIO_OffsetTypeDef;

 typedef struct
{
	 uint8_t cmd;
	 uint8_t data[USB_MAX_EP0_SIZE]  __attribute__ ((aligned(4)));
	 uint8_t len;
	 uint8_t unit;
	 uint16_t req;
} USBD_AUDIO_ControlTypeDef;

typedef struct
{
	uint32_t                  *buffer;  // Ring buffer holding stereo samples
	uint16_t                  rd_ptr;
	uint16_t                  wr_ptr;
	uint32_t                  fs;
	uint32_t                  bytes_per_sample;
	uint32_t                  alt_setting;
	int16_t                   vol;
	AUDIO_StateTypeDef        state;
} USBD_AUDIO_HandleTypeDef;

typedef struct
{
	USBD_AUDIO_ControlTypeDef control;
	USBD_AUDIO_HandleTypeDef  out;
	USBD_AUDIO_HandleTypeDef  in;
} USBD_AUDIO_DeviceTypeDef;

typedef struct
{
  int8_t (*Init)(uint32_t AudioFreq, uint32_t Volume, uint32_t options);
  int8_t (*DeInit)(uint32_t options);
  int8_t (*AudioCmd)(uint8_t *pbuf, uint32_t size, uint8_t cmd);
  int8_t (*VolumeCtl)(uint8_t vol);
  int8_t (*MuteCtl)(uint8_t cmd);
  int8_t (*PeriodicTC)(uint8_t *pbuf, uint32_t size, uint8_t cmd);
  int8_t (*GetState)(void);
} USBD_AUDIO_ItfTypeDef;

#ifdef UAC2
extern USBD_ClassTypeDef UAC2_AUDIO;
#define USBD_AUDIO_CLASS    &UAC2_AUDIO
#else
extern USBD_ClassTypeDef UAC1_AUDIO;
#define USBD_AUDIO_CLASS    &UAC1_AUDIO
#endif
extern uint8_t *uac_cfg_desc;
extern uint16_t uac_cfg_desc_size;

extern SAI_HandleTypeDef hsai_in;	// I2S RX

uint8_t USBD_AUDIO_Init(USBD_HandleTypeDef *pdev, uint8_t cfgidx);
uint8_t USBD_AUDIO_DeInit(USBD_HandleTypeDef *pdev, uint8_t cfgidx);
uint8_t *USBD_AUDIO_GetCfgDesc(uint16_t *length);
uint8_t *USBD_AUDIO_GetDeviceQualifierDesc(uint16_t *length);
uint8_t USBD_AUDIO_RegisterInterface(USBD_HandleTypeDef *pdev, USBD_AUDIO_ItfTypeDef *fops);
uint8_t USBD_AUDIO_DataIn(USBD_HandleTypeDef *pdev, uint8_t epnum);
uint8_t USBD_AUDIO_DataOut(USBD_HandleTypeDef *pdev, uint8_t epnum);
uint8_t USBD_AUDIO_SOF(USBD_HandleTypeDef *pdev);
uint8_t USBD_AUDIO_IsoINIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum);
uint8_t USBD_AUDIO_IsoOutIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum);

void AUDIO_IN_DMA_Sync(USBD_HandleTypeDef *pdev, AUDIO_OffsetTypeDef offset);
void AUDIO_IN_CopyToRingBuffer(USBD_AUDIO_HandleTypeDef *haudio, uint32_t *src, uint16_t size);
void AUDIO_IN_CopySamplesToUsbPacket(uint8_t *dst, uint32_t *src, uint16_t num_samples, uint8_t dst_bytes_per_sample);
void AUDIO_IN_SetSampleRate(USBD_AUDIO_HandleTypeDef *haudio, uint32_t sample_rate);
uint16_t AUDIO_IN_GetPacketSizeInSamples(uint32_t freq, uint16_t counter);
void AUDIO_IN_StopAndReset(USBD_HandleTypeDef* pdev);
void AUDIO_IN_Restart(USBD_HandleTypeDef* pdev);
void AUDIO_IN_StartRecord(uint8_t *buf, uint16_t size);
void AUDIO_IN_StopRecord();

#ifdef __cplusplus
}
#endif

#endif  /* __USB_AUDIO_H */
