/**
  ******************************************************************************
  * @file         stm32f7xx_hal_msp.c
  * @brief        This file provides code for the MSP Initialization
  *               and de-Initialization codes.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

extern DMA_HandleTypeDef hdma_i2c2_rx;
extern DMA_HandleTypeDef hdma_i2c2_tx;

/**
  * Initializes the Global MSP.
  */
void HAL_MspInit(void)
{
	__HAL_RCC_PWR_CLK_ENABLE();
	__HAL_RCC_SYSCFG_CLK_ENABLE();
}

/**
* @brief I2C MSP Initialization
* This function configures the hardware resources used in this example
* @param hi2c: I2C handle pointer
* @retval None
*/
void HAL_I2C_MspInit(I2C_HandleTypeDef* hi2c)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  if (hi2c->Instance == I2C2) {
    __HAL_RCC_GPIOB_CLK_ENABLE();
    /**I2C5 GPIO Configuration
    PB11     ------> I2C2_SCL
    PB10     ------> I2C2_SDA
    */
    GPIO_InitStruct.Pin = I2SIN_SCL_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_MEDIUM;
    GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
    HAL_GPIO_Init(I2SIN_SCL_PORT, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = I2SIN_SDA_PIN;
    HAL_GPIO_Init(I2SIN_SDA_PORT, &GPIO_InitStruct);

    /* Peripheral clock enable */
    __HAL_RCC_I2C2_CLK_ENABLE();

    /* I2C2 DMA Init */
    /* I2C2_RX Init */
    hdma_i2c2_rx.Instance = DMA1_Stream2;
    hdma_i2c2_rx.Init.Channel = DMA_CHANNEL_7;
    hdma_i2c2_rx.Init.Direction = DMA_PERIPH_TO_MEMORY;
    hdma_i2c2_rx.Init.PeriphInc = DMA_PINC_DISABLE;
    hdma_i2c2_rx.Init.MemInc = DMA_MINC_ENABLE;
    hdma_i2c2_rx.Init.PeriphDataAlignment = DMA_PDATAALIGN_BYTE;
    hdma_i2c2_rx.Init.MemDataAlignment = DMA_MDATAALIGN_BYTE;
    hdma_i2c2_rx.Init.Mode = DMA_NORMAL;
    hdma_i2c2_rx.Init.Priority = DMA_PRIORITY_LOW;
    hdma_i2c2_rx.Init.FIFOMode = DMA_FIFOMODE_DISABLE;
    if (HAL_DMA_Init(&hdma_i2c2_rx) != HAL_OK) {
      Error_Handler();
    }
    __HAL_LINKDMA(hi2c,hdmarx,hdma_i2c2_rx);

    /* I2C2_TX Init */
    hdma_i2c2_tx.Instance = DMA1_Stream7;
    hdma_i2c2_tx.Init.Channel = DMA_CHANNEL_7;
    hdma_i2c2_tx.Init.Direction = DMA_MEMORY_TO_PERIPH;
    hdma_i2c2_tx.Init.PeriphInc = DMA_PINC_DISABLE;
    hdma_i2c2_tx.Init.MemInc = DMA_MINC_ENABLE;
    hdma_i2c2_tx.Init.PeriphDataAlignment = DMA_PDATAALIGN_BYTE;
    hdma_i2c2_tx.Init.MemDataAlignment = DMA_MDATAALIGN_BYTE;
    hdma_i2c2_tx.Init.Mode = DMA_NORMAL;
    hdma_i2c2_tx.Init.Priority = DMA_PRIORITY_LOW;
    hdma_i2c2_tx.Init.FIFOMode = DMA_FIFOMODE_DISABLE;
    if (HAL_DMA_Init(&hdma_i2c2_tx) != HAL_OK) {
      Error_Handler();
    }
    __HAL_LINKDMA(hi2c,hdmatx,hdma_i2c2_tx);

    /* I2C5 interrupt Init */
    HAL_NVIC_SetPriority(I2C2_EV_IRQn, 8, 0);
    HAL_NVIC_EnableIRQ(I2C2_EV_IRQn);
    HAL_NVIC_SetPriority(I2C2_ER_IRQn, 8, 0);
    HAL_NVIC_EnableIRQ(I2C2_ER_IRQn);
   }

}

/**
* @brief I2C MSP De-Initialization
* This function freeze the hardware resources used in this example
* @param hi2c: I2C handle pointer
* @retval None
*/
void HAL_I2C_MspDeInit(I2C_HandleTypeDef* hi2c)
{
	if (hi2c->Instance == I2C2) {
		/* Peripheral clock disable */
		__HAL_RCC_I2C2_CLK_DISABLE();

		/**I2C2 GPIO Configuration
    	PB10     ------> I2C2_SDA
    	PB11     ------> I2C2_SCL
		*/
		HAL_GPIO_DeInit(I2SIN_SCL_PORT, I2SIN_SCL_PIN);
		HAL_GPIO_DeInit(I2SIN_SDA_PORT, I2SIN_SDA_PIN);

		/* I2C2 DMA DeInit */
		HAL_DMA_DeInit(hi2c->hdmarx);
		HAL_DMA_DeInit(hi2c->hdmatx);

		/* I2C2 interrupt DeInit */
		HAL_NVIC_DisableIRQ(I2C2_EV_IRQn);
		HAL_NVIC_DisableIRQ(I2C2_ER_IRQn);
	}
}

/**
* @brief TIM_Base MSP Initialization
* This function configures the hardware resources used in this example
* @param htim_base: TIM_Base handle pointer
* @retval None
*/
void HAL_TIM_Base_MspInit(TIM_HandleTypeDef* htim_base)
{
	if(htim_base->Instance == TIM11) {
	    /* Peripheral clock enable */
	    __HAL_RCC_TIM11_CLK_ENABLE();
	    /* TIM11 interrupt Init */
		HAL_NVIC_SetPriority(TIM1_TRG_COM_TIM11_IRQn, 8, 0);
		HAL_NVIC_EnableIRQ(TIM1_TRG_COM_TIM11_IRQn);
	}
	else if (htim_base->Instance == TIM10) {
		/* Peripheral clock enable */
		__HAL_RCC_TIM10_CLK_ENABLE();
		/* TIM10 interrupt Init */
		HAL_NVIC_SetPriority(TIM1_UP_TIM10_IRQn, 8, 0);
		HAL_NVIC_EnableIRQ(TIM1_UP_TIM10_IRQn);
	}
}

void HAL_TIM_MspPostInit(TIM_HandleTypeDef* htim)
{
#ifdef X
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	if (htim->Instance == TIM10) {
		__HAL_RCC_GPIOB_CLK_ENABLE();
		/**TIM10 GPIO Configuration
    	PB8     ------> TIM10_CH1
		 */
		GPIO_InitStruct.Pin = GPIO_PIN_8;
		GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
		GPIO_InitStruct.Pull = GPIO_NOPULL;
		GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
		GPIO_InitStruct.Alternate = GPIO_AF3_TIM10;
		HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
	}
#endif
}

/**
* @brief TIM_Base MSP De-Initialization
* This function freeze the hardware resources used in this example
* @param htim_base: TIM_Base handle pointer
* @retval None
*/
void HAL_TIM_Base_MspDeInit(TIM_HandleTypeDef* htim_base)
{
	if(htim_base->Instance == TIM11) {
	    /* Peripheral clock disable */
	    __HAL_RCC_TIM11_CLK_DISABLE();

	    /* TIM11 interrupt DeInit */
		HAL_NVIC_DisableIRQ(TIM1_TRG_COM_TIM11_IRQn);
	}
	else if (htim_base->Instance == TIM10) {
		/* Peripheral clock disable */
		__HAL_RCC_TIM10_CLK_DISABLE();

		/* TIM10 interrupt DeInit */
		HAL_NVIC_DisableIRQ(TIM1_UP_TIM10_IRQn);
	}
}

extern SAI_HandleTypeDef hsai_in;
extern DMA_HandleTypeDef hdma_sai_in;

static uint32_t SAI2_client = 0;

void HAL_SAI_MspInit(SAI_HandleTypeDef* hsai)
{
	GPIO_InitTypeDef GPIO_InitStruct;
	/* SAI2 */
    if (hsai->Instance == hsai_in.Instance) {
    	/* Peripheral clock enable */
    	if (SAI2_client == 0) {
    		__HAL_RCC_SAI2_CLK_ENABLE();
    	}
    	SAI2_client ++;

    	/**SAI2_Block_B GPIO Configuration
    	PE13     ------> SAI2_FS_B
    	PE12     ------> SAI2_SCK_B
    	PE11     ------> SAI2_SD_B
    	 */
    	GPIO_InitStruct.Pin = I2SIN_FS_PIN;
    	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    	GPIO_InitStruct.Pull = GPIO_NOPULL;
    	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
    	GPIO_InitStruct.Alternate = GPIO_AF10_SAI2;
    	HAL_GPIO_Init(I2SIN_FS_PORT, &GPIO_InitStruct);

    	GPIO_InitStruct.Pin = I2SIN_SCK_PIN;
    	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
    	HAL_GPIO_Init(I2SIN_SCK_PORT, &GPIO_InitStruct);
    	GPIO_InitStruct.Pin = I2SIN_SD_PIN;
    	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
    	HAL_GPIO_Init(I2SIN_SD_PORT, &GPIO_InitStruct);

    	/* Peripheral DMA init*/
        hdma_sai_in.Instance = DMA2_Stream6;
        hdma_sai_in.Init.Channel = DMA_CHANNEL_3;
        hdma_sai_in.Init.Direction = DMA_PERIPH_TO_MEMORY;
        hdma_sai_in.Init.PeriphInc = DMA_PINC_DISABLE;
        hdma_sai_in.Init.MemInc = DMA_MINC_ENABLE;
    	hdma_sai_in.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
    	hdma_sai_in.Init.MemDataAlignment = DMA_MDATAALIGN_WORD;
    	hdma_sai_in.Init.Mode = DMA_CIRCULAR;
    	hdma_sai_in.Init.Priority = DMA_PRIORITY_HIGH;
    	hdma_sai_in.Init.FIFOMode = DMA_FIFOMODE_ENABLE;
    	hdma_sai_in.Init.FIFOThreshold = DMA_FIFO_THRESHOLD_FULL;
    	hdma_sai_in.Init.MemBurst = DMA_MBURST_SINGLE;
    	hdma_sai_in.Init.PeriphBurst = DMA_PBURST_SINGLE;
    	if (HAL_DMA_Init(&hdma_sai_in) != HAL_OK) {
    		Error_Handler();
    	}
    	/* Several peripheral DMA handle pointers point to the same DMA handle.
     	 Be aware that there is only one channel to perform all the requested DMAs. */
    	__HAL_LINKDMA(hsai,hdmarx,hdma_sai_in);
    	__HAL_LINKDMA(hsai,hdmatx,hdma_sai_in);
    }
}

void HAL_SAI_MspDeInit(SAI_HandleTypeDef* hsai)
{
	/* SAI2 */
    if (hsai->Instance == SAI2_Block_B) {
    	SAI2_client --;
    	if (SAI2_client == 0) {
    		/* Peripheral clock disable */
    		__HAL_RCC_SAI2_CLK_DISABLE();
    	}

    	/**SAI2_Block_B GPIO Configuration
    	PE13     ------> SAI2_FS_B
    	PE12     ------> SAI2_SCK_B
    	PE11     ------> SAI2_SD_B
    	*/
    	HAL_GPIO_DeInit(I2SIN_FS_PORT, I2SIN_FS_PIN);
    	HAL_GPIO_DeInit(I2SIN_SCK_PORT, I2SIN_SCK_PIN);
    	HAL_GPIO_DeInit(I2SIN_SD_PORT, I2SIN_SD_PIN);

    	HAL_DMA_DeInit(hsai->hdmarx);
    	HAL_DMA_DeInit(hsai->hdmatx);
    }
}
