/**
  ******************************************************************************
  * @file    uac2_audio.c
  * @author  MK
  * @version V0.2
  * @date    08/01/2021
  * @brief   This file provides the USB Audio Class 2 (UAC2) core functions.
  *
  * @verbatim
  *      
  *          ===================================================================      
  *                                AUDIO Class  Description
  *          ===================================================================
  *           This driver manages the Audio Class 2.0 following the "USB Device Class Definition for
  *           Audio Devices V2.0 May 31, 2006".
  *
  ******************************************************************************
  */ 
#include "main.h"
#include "usbd_audio.h"
#include "uac2_descriptor.h"
#include "DWT_delay.h"
#include "queue_cmd.h"
#include <stdbool.h>

#define NUM_SAMPLE_RATES_IN  10
const uint32_t sample_rates_in[]  = {44100L, 48000L, 88200L, 96000L, 176400L, 192000L, 352800L, 384000L, 705600L, 768000L};

static uint8_t UAC2_AUDIO_Setup(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);
static uint8_t UAC2_AUDIO_EP0_RxReady(USBD_HandleTypeDef *pdev);
static uint8_t UAC2_AUDIO_EP0_TxReady(USBD_HandleTypeDef *pdev);
static void UAC2_REQ_GetCurrent(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);
static void UAC2_REQ_SetCurrent(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);
static void UAC2_REQ_GetRange(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);
static void UAC2_REQ_SetRange(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);

USBD_ClassTypeDef  UAC2_AUDIO =
{
  USBD_AUDIO_Init,
  USBD_AUDIO_DeInit,
  UAC2_AUDIO_Setup,
  UAC2_AUDIO_EP0_TxReady,
  UAC2_AUDIO_EP0_RxReady,
  USBD_AUDIO_DataIn,
  USBD_AUDIO_DataOut,
  USBD_AUDIO_SOF,
  USBD_AUDIO_IsoINIncomplete,
  USBD_AUDIO_IsoOutIncomplete,
  USBD_AUDIO_GetCfgDesc,
  USBD_AUDIO_GetCfgDesc,
  USBD_AUDIO_GetCfgDesc,
  USBD_AUDIO_GetDeviceQualifierDesc,
};

/**
  * @brief  UAC2_AUDIO_Setup
  *         Handle the AUDIO specific requests
  * @param  pdev: instance
  * @param  req: usb requests
  * @retval status
  */
static uint8_t  UAC2_AUDIO_Setup (USBD_HandleTypeDef *pdev,
                                USBD_SetupReqTypedef *req)
{
	USBD_AUDIO_HandleTypeDef *haudio;
	uint16_t len;
	uint8_t *pbuf;
	uint16_t status_info = 0U;
	uint8_t ret = USBD_OK;

	if (_cfg.debug) {
		printf("Setup %02x %02x %04x %04x %d %d %ld\r\n", req->bmRequest, req->bRequest, req->wIndex, req->wValue, req->wLength, pdev->dev_state, DWT_GetTick());
	}
	switch (req->bmRequest & USB_REQ_TYPE_MASK) {
		case USB_REQ_TYPE_CLASS :
			switch (req->bRequest) {
				case UAC2_CS_REQ_CUR:
					if (req->bmRequest & 0x80) {
						UAC2_REQ_GetCurrent(pdev, req);
					}
					else {
						UAC2_REQ_SetCurrent(pdev, req);
					}
					break;

				case UAC2_CS_REQ_RANGE:
					if (req->bmRequest & 0x80) {
						UAC2_REQ_GetRange(pdev, req);
					}
					else {
						UAC2_REQ_SetRange(pdev, req);
					}
					break;

				default:
					USBD_CtlError (pdev, req);
					ret = USBD_FAIL;
					break;
			}
			break;

		case USB_REQ_TYPE_STANDARD:
			switch (req->bRequest) {
				case USB_REQ_GET_STATUS:
					if (pdev->dev_state == USBD_STATE_CONFIGURED) {
						(void)USBD_CtlSendData(pdev, (uint8_t *)&status_info, 2U);
					}
					else {
						USBD_CtlError(pdev, req);
						ret = USBD_FAIL;
					}
					break;

				case USB_REQ_GET_DESCRIPTOR:
					if ((req->wValue >> 8) == UAC2_CS_DEVICE) {
						pbuf = uac_cfg_desc + 18;
						len = MIN(uac_cfg_desc_size, req->wLength);
						if (_cfg.debug) {
							printf("GetDesc: %d %d\r\n", uac_cfg_desc_size, req->wLength);
						}
						USBD_CtlSendData (pdev, pbuf, len);
					}
					break;

				case USB_REQ_GET_INTERFACE :
					if (pdev->dev_state == USBD_STATE_CONFIGURED) {
						if (req->wIndex <= USBD_MAX_NUM_INTERFACES) {
							if (req->wIndex == 1) {
								(void)USBD_CtlSendData(pdev, (uint8_t *)&((USBD_AUDIO_DeviceTypeDef*)pdev->pClassData)->in.alt_setting, 1U);
							}
						}
					}
					else {
						USBD_CtlError(pdev, req);
						ret = USBD_FAIL;
					}
					break;

				case USB_REQ_SET_INTERFACE :
					if (pdev->dev_state == USBD_STATE_CONFIGURED) {
						if (req->wIndex <= USBD_MAX_NUM_INTERFACES) {
							if (_cfg.debug) {
								printf("SetIntf: %d %d %ld\r\n", req->wIndex, req->wValue, DWT_GetTick());
							}
							if (req->wIndex == 1) {
								haudio = &((USBD_AUDIO_DeviceTypeDef*)pdev->pClassData)->in;
								haudio->alt_setting = (uint8_t)(req->wValue);
								if (haudio->alt_setting == 0U) {
									AUDIO_IN_StopAndReset(pdev);
								}
#if (CFG_I2S_IN_16BIT == 1)
								else if (haudio->alt_setting == IN_ALT_SETTING_16BIT) {
									haudio->bytes_per_sample = 2;
									AUDIO_IN_Restart(pdev);
								}
#endif
#if (CFG_I2S_IN_24BIT == 1)
								else if (haudio->alt_setting == IN_ALT_SETTING_24BIT) {
									haudio->bytes_per_sample = 3;
									AUDIO_IN_Restart(pdev);
								}
#endif
#if (CFG_I2S_IN_32BIT == 1)
								else if (haudio->alt_setting == IN_ALT_SETTING_32BIT) {
									haudio->bytes_per_sample = 4;
									AUDIO_IN_Restart(pdev);
								}
#endif
							}
						}
						else {
							/* Call the error management function (command will be nacked */
							USBD_CtlError(pdev, req);
							ret = USBD_FAIL;
						}
					}
					else {
						USBD_CtlError(pdev, req);
						ret = USBD_FAIL;
					}
					break;

				case USB_REQ_CLEAR_FEATURE:
					break;

			    default:
			    	USBD_CtlError(pdev, req);
			    	ret = USBD_FAIL;
			    	break;
			}
			break;
		default:
			USBD_CtlError(pdev, req);
			ret = USBD_FAIL;
			break;
	}
	return (uint8_t)ret;
}

/**
  * @brief  UAC2_AUDIO_EP0_RxReady
  *         handle EP0 Rx Ready event
  * @param  pdev: device instance
  * @retval status
  */
static uint8_t  UAC2_AUDIO_EP0_RxReady (USBD_HandleTypeDef *pdev)
{
	USBD_AUDIO_ControlTypeDef *control;

	control = &((USBD_AUDIO_DeviceTypeDef *)pdev->pClassData)->control;
	if (_cfg.debug) {
		printf("RxReady: %02x %d %02x %04x ", control->cmd, control->len, control->unit, control->req);
		for (int i = 0; i < control->len; i++)
			printf("%02x", control->data[i]);
		printf("\r\n");
	}
	/* Check if an AudioControl request has been issued */
	if (control->cmd == UAC2_CS_REQ_CUR) {
		/* In this driver, to simplify code, only SET_CUR request is managed */
		if (control->unit == UAC2_ENTITY_CLOCK_IN) {
			if (HIBYTE(control->req) == UAC2_CS_CTRL_SAMPLING_FREQ) {
				if (control->len == 4) {
					sendQueueCmd(audio_in_queue, FS, control->data);
					AUDIO_IN_SetSampleRate(&((USBD_AUDIO_DeviceTypeDef *)pdev->pClassData)->in, *(uint32_t *)&control->data[0]);
				}
			}
		}
	}
	return (uint8_t)USBD_OK;
}

/**
  * @brief  UAC2_AUDIO_EP0_TxReady
  *         handle EP0 TRx Ready event
  * @param  pdev: device instance
  * @retval status
  */
static uint8_t UAC2_AUDIO_EP0_TxReady(USBD_HandleTypeDef *pdev)
{
	UNUSED(pdev);

	/* Only OUT control data are processed */
	return (uint8_t)USBD_OK;
}


/**
  * @brief  UAC2_Req_GetCurrent
  *         Handles the GET_CUR Audio control request.
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void UAC2_REQ_GetCurrent(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req)
{
	USBD_AUDIO_HandleTypeDef *haudio;
	if (_cfg.debug) {
		printf("GetCurrent: %02x %04x %04x %d %ld\r\n", req->bmRequest, req->wIndex, req->wValue, req->wLength, DWT_GetTick());
	}
	if ((req->bmRequest & 0x0f) == UAC2_INTERFACE_SUBCLASS_CONTROL) {
		switch (HIBYTE(req->wIndex)) {
			case UAC2_ENTITY_CLOCK_IN:
				if (HIBYTE(req->wValue) == UAC2_CS_CTRL_SAMPLING_FREQ) {
					/* Send the current sampling frequency */
					haudio = &((USBD_AUDIO_DeviceTypeDef *)pdev->pClassData)->in;
					audio_control_cur_4_t current_fs __attribute__ ((aligned(4))) = { .bCur = haudio->fs };
					if (_cfg.debug) {
						printf("Send IN sampling frequency: %ld %ld\r\n", current_fs.bCur, DWT_GetTick());
					}
					USBD_CtlSendData(pdev, (uint8_t *)&current_fs, sizeof(current_fs));
				}
				else  if (HIBYTE(req->wValue) == UAC2_CS_CTRL_CLK_VALID) {
		        	/* Send clock validity */
					if (_cfg.debug) {
						printf("Send IN clock validity %ld\r\n", DWT_GetTick());
					}
					audio_control_cur_1_t clk_valid __attribute__ ((aligned(4))) = { .bCur = 1 };
					USBD_CtlSendData(pdev, (uint8_t *)&clk_valid, sizeof(clk_valid));
		    	}
				break;
			default:
				break;
		}
    }
}

/**
  * @brief  UAC2_Req_SetCurrent
  *         Handles the SET_CUR Audio control request.
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void UAC2_REQ_SetCurrent(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req)
{
	USBD_AUDIO_ControlTypeDef *control;
	control = &((USBD_AUDIO_DeviceTypeDef *)pdev->pClassData)->control;

	if (_cfg.debug) {
		printf("SetCurrent: %02x %d %04x %04x\r\n", req->bmRequest, req->wLength, req->wIndex, req->wValue);
	}
	if (req->wLength != 0U) {
		/* Prepare the reception of the buffer over EP0 */
		(void)USBD_CtlPrepareRx(pdev, control->data, req->wLength);

		control->cmd = UAC2_CS_REQ_CUR;       /* Set the request value */
		control->len = (uint8_t)req->wLength; /* Set the request data length */
		control->unit = HIBYTE(req->wIndex);  /* Set the request target unit */
		control->req = (req->wValue & 0xff00) | req->bmRequest;
	}
}

/**
  * @brief  UAC2_Req_GetRange
  *         Handles the GET_RANGE Audio control request.
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void UAC2_REQ_GetRange(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req)
{
	if (_cfg.debug) {
		printf("GetRange: %02x %04x %04x %d\r\n", req->bmRequest, req->wIndex, req->wValue, req->wLength);
	}
	if ((req->bmRequest & 0x0f) == UAC2_INTERFACE_SUBCLASS_CONTROL) {
		switch (HIBYTE(req->wIndex)) {

		case UAC2_ENTITY_CLOCK_IN:
				if (HIBYTE(req->wValue) == UAC2_CS_CTRL_SAMPLING_FREQ) {
		        	/* Send sampling frequency ranges */
					static audio_control_range_4_n_t(NUM_SAMPLE_RATES_IN) rangef __attribute__ ((aligned(4))) =
					{
						.wNumSubRanges = NUM_SAMPLE_RATES_IN,
					};
					for(uint8_t i = 0; i < NUM_SAMPLE_RATES_IN; i++) {
						rangef.subrange[i].bMin = sample_rates_in[i];
						rangef.subrange[i].bMax = sample_rates_in[i];
						rangef.subrange[i].bRes = 0;
						if (_cfg.debug) {
							printf("Range %d (%d, %d, %d) %d\r\n", i, (int)rangef.subrange[i].bMin, (int)rangef.subrange[i].bMax, (int)rangef.subrange[i].bRes, sizeof(rangef));
						}
					}
					USBD_CtlSendData(pdev, (uint8_t *)&rangef, (req->wLength < sizeof(rangef)) ? req->wLength : sizeof(rangef));
				}
				break;
			default:
				break;
		}
	}
}

/**
  * @brief  UAC2_Req_SetRange
  *         Handles the SET_RANGE Audio control request.
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void UAC2_REQ_SetRange(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req)
{
	USBD_AUDIO_ControlTypeDef *control;
	control = &((USBD_AUDIO_DeviceTypeDef *)pdev->pClassData)->control;

	if (_cfg.debug) {
		printf("SetRange: %02x %d %04x %04x\r\n", req->bmRequest, req->wLength, req->wIndex, req->wValue);
	}
	if (req->wLength != 0U) {
		/* Prepare the reception of the buffer over EP0 */
		(void)USBD_CtlPrepareRx(pdev, control->data, req->wLength);

		control->cmd = UAC2_CS_REQ_RANGE;     /* Set the request value */
		control->len = (uint8_t)req->wLength; /* Set the request data length */
		control->unit = HIBYTE(req->wIndex);  /* Set the request target unit */
		control->req = (req->wValue & 0xff00) | req->bmRequest;
	}
}
