/**
  ******************************************************************************
  * @file    uac2_descriptor.c
  * @author  MK
  * @version V0.2
  * @date    08/01/2021
  * @brief   USD Audio Class 2 device configuration descriptors.
  ******************************************************************************
  */ 

#include "usbd_audio.h"
#include "uac2_descriptor.h"

#define U32_BYTE3(u32)     ((uint8_t) ((((uint32_t) u32) >> 24) & 0x000000ff)) // MSB
#define U32_BYTE2(u32)     ((uint8_t) ((((uint32_t) u32) >> 16) & 0x000000ff))
#define U32_BYTE1(u32)     ((uint8_t) ((((uint32_t) u32) >>  8) & 0x000000ff))
#define U32_BYTE0(u32)     ((uint8_t) (((uint32_t)  u32)        & 0x000000ff)) // LSB
#define U32_TO_U8S_BE(u32) U32_BYTE3(u32), U32_BYTE2(u32), U32_BYTE1(u32), U32_BYTE0(u32)
#define U32_TO_U8S_LE(u32) U32_BYTE0(u32), U32_BYTE1(u32), U32_BYTE2(u32), U32_BYTE3(u32)

enum
{
  ITF_NUM_AUDIO_CONTROL = 0,
#if (CFG_I2S_OUT == 1)
  ITF_NUM_AUDIO_STREAMING_SPK,
#endif
#if (CFG_I2S_IN == 1)
  ITF_NUM_AUDIO_STREAMING_MIC,
#endif
  ITF_NUM_TOTAL
};

/* USB AUDIO CLASS 2 Device Configuration Descriptor */
__ALIGN_BEGIN uint8_t UAC2_CfgDesc[UAC2_CONFIG_DESC_SIZE] __ALIGN_END =
{
	/* Configuration 1 */
	// 09 02 41 01 03 01 00 a0 32
	USB_LEN_CFG_DESC,                     /* bLength */
	USB_DESC_TYPE_CONFIGURATION,          /* bDescriptorType */
	LOBYTE(UAC2_CONFIG_DESC_SIZE),        /* wTotalLength bytes*/
	HIBYTE(UAC2_CONFIG_DESC_SIZE),
	ITF_NUM_TOTAL,                        /* bNumInterfaces */
	0x01,                                 /* bConfigurationValue */
	0x00,                                 /* iConfiguration */
	0x80,                                 /* bmAttributes  0xc0 Self-powered, no remote wakeup*/
	0x32,                                 /* bMaxPower = 100 mA*/
	/* 09 byte*/

	/* Standard Interface Association Descriptor (IAD) */
	// 08 0b 00 03 01 00 20 00
	UAC2_STD_IAD_SIZE,                    /* bLength */
	USB_DESC_TYPE_INTERFACE_ASSOCIATION,  /* bDescriptorType */
	ITF_NUM_AUDIO_CONTROL,                /* bFirstInterface */
	ITF_NUM_TOTAL,                        /* bInterfaceCount */
	UAC2_FUNCTION_CLASS_AUDIO,            /* bFunctionClass */
	UAC2_FUNCTION_SUBCLASS_UNDEFINED,     /* bFunctionSubClass */
	UAC2_FUNC_PROTOCOL_CODE_V2,           /* bFunctionProtocol */
	0x00,                                 /* iFunction */
	/* 08 byte*/

	/* Standard AC Interface Descriptor(4.7.1) */
	// 09 04 00 00 00 01 01 20 02
	UAC2_INTERFACE_DESC_SIZE,             /* bLength */
	USB_DESC_TYPE_INTERFACE,              /* bDescriptorType */
	ITF_NUM_AUDIO_CONTROL,                /* bInterfaceNumber */
	0x00,                                 /* bAlternateSetting */
	0x00,                                 /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,           /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_CONTROL,      /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,      /* bInterfaceProtocol */
	0x00,                                 /* iInterface */
    /* 09 byte*/

	/* Class-Specific AC Interface Header Descriptor(4.7.2) */
	// 09 24 01 00 02 04 5d 00 00
	UAC2_CS_AC_INTERFACE_DESC_SIZE,       /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_HEADER,          /* bDescriptorSubType */
	0x00,                                 /* bcdADC */
	0x02,
	UAC2_FUNC_IO_BOX,                     /* bCategory */
	LOBYTE(UAC2_CS_AC_INTERFACE_TOTAL_LENGTH), /* wTotalLength */
	HIBYTE(UAC2_CS_AC_INTERFACE_TOTAL_LENGTH),
	0x00,                                 /* bmControls */
    /* 09 byte*/

#if (CFG_I2S_OUT == 1)
	/* Clock Source Descriptor(4.7.2.1) */
	// 08 24 0a 04 03 07 00 00
	UAC2_CLK_SRC_DESC_SIZE,               /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_CLOCK_SOURCE,    /* bDescriptorSubType */
	UAC2_ENTITY_CLOCK_OUT,                /* bClockID */
	0x01,                                 /* bmAttributes 0x03*/
	0x07,                                 /* bmControls 0x07*/
//	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,       /* bAssocTerminal */
	0x00,                                 /* iClockSource */
    /* 08 byte*/
#endif

#if (CFG_I2S_IN == 1)
	/* Clock Source Descriptor(4.7.2.1) */
	// 08 24 0a 04 03 07 00 00
	UAC2_CLK_SRC_DESC_SIZE,               /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_CLOCK_SOURCE,    /* bDescriptorSubType */
	UAC2_ENTITY_CLOCK_IN,                 /* bClockID */
	0x01,                                 /* bmAttributes 0x03*/
	0x07,                                 /* bmControls 0x07*/
//	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_MIC_INPUT_TERMINAL,       /* bAssocTerminal */
	0x00,                                 /* iClockSource */
    /* 08 byte*/
#endif

#if (CFG_I2S_OUT == 1)
	/* Input Terminal Descriptor(4.7.2.4) */
	// 11 24 02 01 01 01 00 04 02 00 00 00 00 00 00 00 00
	UAC2_INPUT_TERMINAL_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_INPUT_TERMINAL,  /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,       /* bTerminalID */
	LOBYTE(UAC2_TERM_TYPE_USB_STREAMING), /* wTerminalType */
	HIBYTE(UAC2_TERM_TYPE_USB_STREAMING),
	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_CLOCK_OUT,                /* bSourceID */
	0x02,                                 /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                 /* iChannelNames */
	0x00,                                 /* bmControls */
	0x00,
	0x00,                                 /* iTerminal */
	/* 17 byte*/

	/* Feature Unit Descriptor(4.7.2.8) */
	// 12 24 06 02 01 0f 00 00 00 0f 00 00 00 0f 00 00 00 00
	0x12,                                 /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_FEATURE_UNIT,    /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_FEATURE_UNIT,         /* bUnitID */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,       /* bSourceID */
#ifdef X
	U32_TO_U8S_LE(UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_MUTE_POS | UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_VOLUME_POS),  /* bmaControls(0) */
	U32_TO_U8S_LE(UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_MUTE_POS | UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_VOLUME_POS),  /* bmaControls(1) */
	U32_TO_U8S_LE(UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_MUTE_POS | UAC2_CTRL_RW << UAC2_FEATURE_UNIT_CTRL_VOLUME_POS),  /* bmaControls(2) */
#else
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
#endif
	0x00,                                 /* iFeature */
	/* 18 byte*/

	/* Output Terminal Descriptor(4.7.2.5) */
	// 0c 24 03 03 02 03 00 02 04 00 00 00
	UAC2_OUTPUT_TERMINAL_DESC_SIZE,       /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_OUTPUT_TERMINAL, /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_OUTPUT_TERMINAL,      /* bTerminalID */
	LOBYTE(UAC2_TERM_TYPE_EXT_LINE_CONNECTOR), /* wTerminalType */
	HIBYTE(UAC2_TERM_TYPE_EXT_LINE_CONNECTOR),
	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_SPK_FEATURE_UNIT,         /* bSourceID */
//	UAC2_ENTITY_SPK_INPUT_TERMINAL,         /* bSourceID */
	UAC2_ENTITY_CLOCK_OUT,                /* bCSourceID */
	0x00,                                 /* bmControls */
	0x00,
	0x00,                                 /* iTerminal */
	/* 12 byte*/
#endif

#if (CFG_I2S_IN == 1)
	/* Input Terminal Descriptor(4.7.2.4) */
	// 11 24 02 11 01 02 00 04 01 00 00 00 00 00 00 00 00
    UAC2_INPUT_TERMINAL_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_INPUT_TERMINAL,  /* bDescriptorSubtype */
	UAC2_ENTITY_MIC_INPUT_TERMINAL,       /* bTerminalID */
	LOBYTE(UAC2_TERM_TYPE_EXT_LINE_CONNECTOR), /* wTerminalType */
	HIBYTE(UAC2_TERM_TYPE_EXT_LINE_CONNECTOR),
	UAC2_ENTITY_MIC_OUTPUT_TERMINAL,      /* bAssocTerminal */
//	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_CLOCK_IN,                 /* bSourceID */
	0x02,                                 /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                 /* iChannelNames */
	0x00,                                 /* bmControls */
	0x00,
	0x00,                                 /* iTerminal */
	/* 17 byte*/

	/* Output Terminal Descriptor(4.7.2.5) */
	// 0c 24 03 13 01 01 00 11 04 00 00 00
    UAC2_OUTPUT_TERMINAL_DESC_SIZE,       /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AC_INTERFACE_OUTPUT_TERMINAL, /* bDescriptorSubtype */
	UAC2_ENTITY_MIC_OUTPUT_TERMINAL,      /* bTerminalID */
	LOBYTE(UAC2_TERM_TYPE_USB_STREAMING), /* wTerminalType */
	HIBYTE(UAC2_TERM_TYPE_USB_STREAMING),
	UAC2_ENTITY_MIC_INPUT_TERMINAL,       /* bAssocTerminal */
//	0x00,                                 /* bAssocTerminal */
	UAC2_ENTITY_MIC_INPUT_TERMINAL,       /* bSourceID */
	UAC2_ENTITY_CLOCK_IN,                 /* bCSourceID */
	0x00,                                 /* bmControls */
	0x00,
	0x00,                                 /* iTerminal */
	/* 12 byte*/
#endif

#if (CFG_I2S_OUT == 1)
	/* Output interface */
    /* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 1, Alternate 0 - default alternate setting with 0 bandwidth */
	// 09 04 01 00 00 01 02 20 05
	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_SPK,           /* bInterfaceNumber */
	0x00,                                  /* bAlternateSetting */
	0x00,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

#if (CFG_I2S_OUT_16BIT == 1)
    /* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 1, Alternate 1 - alternate interface for data streaming */
	// 09 04 01 01 01 01 02 20 05
    UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_SPK,           /* bInterfaceNumber */
	OUT_ALT_SETTING_16BIT,                 /* bAlternateSetting */
	0x02,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	// 10 24 01 01 00 01 01 00 00 00 02 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	0x01,                                  /* bmFormats UAC2_DATA_FORMAT_TYPE_I_PCM */
	0x00,
	0x00,
	0x00,
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	// 06 24 02 01 02 10
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x02,                                 /* bSubSlotSizes */
	0x10,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	// 07 05 01 09 84 01 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
//	LOBYTE(USB_BUFFER_SIZE_16),           /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
//	HIBYTE(USB_BUFFER_SIZE_16),
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	// 08 25 01 00 00 01 01 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	0x00,                                 /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/

	/* ##Endpoint 2 for feedback - Standard Descriptor */
	UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE, /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_FB_EP,                      /* bEndpointAddress 2 in endpoint*/
	0x11,                                 /* bmAttributes */
	LOBYTE(FB_PACKET_SIZE),               /* wMaxPacketSize in bytes */
	HIBYTE(FB_PACKET_SIZE),
	FB_REFRESH,                           /* bInterval */
	/* 07 byte*/
#endif

#if (CFG_I2S_OUT_24BIT == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 1, Alternate 2 - alternate interface for data streaming */
	/*09 04 01 02 01 01 02 20 05*/
 	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_SPK,           /* bInterfaceNumber */
	OUT_ALT_SETTING_24BIT,                 /* bAlternateSetting */
	0x02,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	// 10 24 01 01 00 01 01 00 00 00 02 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	0x01,                                  /* bmFormats UAC2_DATA_FORMAT_TYPE_I_PCM */
	0x00,
	0x00,
	0x00,
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	// 06 24 02 01 04 18
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x03,                                 /* bSubSlotSizes */
	0x18,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	// 07 05 01 09 08 03 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
//	LOBYTE(USB_BUFFER_SIZE_24),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
//	HIBYTE(USB_BUFFER_SIZE_24),
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	// 08 25 01 00 00 01 01 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/

	/* ##Endpoint 2 for feedback - Standard Descriptor */
	UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE, /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_FB_EP,                      /* bEndpointAddress 2 in endpoint*/
	0x11,                                 /* bmAttributes */
	LOBYTE(FB_PACKET_SIZE),               /* wMaxPacketSize in bytes */
	HIBYTE(FB_PACKET_SIZE),
	FB_REFRESH,                           /* bInterval */
	/* 07 byte*/
#endif

#if (CFG_I2S_OUT_32BIT == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 1, Alternate 3 - alternate interface for data streaming */
	/*09 04 01 02 01 01 02 20 05*/
 	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_SPK,           /* bInterfaceNumber */
	OUT_ALT_SETTING_32BIT,                 /* bAlternateSetting */
	0x02,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	// 10 24 01 01 00 01 01 00 00 00 02 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	U32_TO_U8S_LE(UAC2_DATA_FORMAT_TYPE_I_PCM),    /* bmFormats UAC2_DATA_FORMAT_TYPE_RAW_DATA */
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	// 06 24 02 01 04 18
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x04,                                 /* bSubSlotSizes */
	0x20,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	// 07 05 01 09 08 03 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	// 08 25 01 00 00 01 01 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/

	/* ##Endpoint 2 for feedback - Standard Descriptor */
	UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE, /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_FB_EP,                      /* bEndpointAddress 2 in endpoint*/
	0x11,                                 /* bmAttributes */
	LOBYTE(FB_PACKET_SIZE),               /* wMaxPacketSize in bytes */
	HIBYTE(FB_PACKET_SIZE),
	FB_REFRESH,                           /* bInterval */
	/* 07 byte*/
#endif

#if (CFG_I2S_OUT_RAW == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 1, Alternate 4 - alternate interface for data streaming */
	/*09 04 01 02 01 01 02 20 05*/
 	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_SPK,           /* bInterfaceNumber */
	OUT_ALT_SETTING_RAW,                   /* bAlternateSetting */
	0x02,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	// 10 24 01 01 00 01 01 00 00 00 02 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_SPK_INPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	U32_TO_U8S_LE(UAC2_DATA_FORMAT_TYPE_I_RAW_DATA),    /* bmFormats UAC2_DATA_FORMAT_TYPE_RAW_DATA */
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	// 06 24 02 01 04 18
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x04,                                 /* bSubSlotSizes */
	0x20,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	// 07 05 01 09 08 03 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	// 08 25 01 00 00 01 01 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/

	/* ##Endpoint 2 for feedback - Standard Descriptor */
	UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE, /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_OUT_FB_EP,                      /* bEndpointAddress 2 in endpoint*/
	0x11,                                 /* bmAttributes */
	LOBYTE(FB_PACKET_SIZE),               /* wMaxPacketSize in bytes */
	HIBYTE(FB_PACKET_SIZE),
	FB_REFRESH,                           /* bInterval */
	/* 07 byte*/
#endif
#endif

#if (CFG_I2S_IN == 1)
	/**** Input interface ****/
	/* Standard AS Interface Descriptor(4.9.1) */
	/* Interface 2, Alternate 0 - default alternate setting with 0 bandwidth */
	//09 04 02 00 00 01 02 20 04
 	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_MIC,           /* bInterfaceNumber */
	0x00,                                  /* bAlternateSetting */
	0x00,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

#if (CFG_I2S_IN_16BIT == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
	/* Interface 2, Alternate 1 - alternate interface for data streaming */
	//09 04 02 01 01 01 02 20 04
    UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_MIC,           /* bInterfaceNumber */
	IN_ALT_SETTING_16BIT,                  /* bAlternateSetting */
	0x01,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	//10 24 01 13 00 01 01 00 00 00 01 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_MIC_OUTPUT_TERMINAL,       /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	0x01,                                  /* bmFormats UAC2_DATA_FORMAT_TYPE_I_PCM */
	0x00,
	0x00,
	0x00,
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	//06 24 02 01 02 10
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x02,                                 /* bSubSlotSizes */
	0x10,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	//07 05 81 05 c2 00 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_IN_EP,                          /* bEndpointAddress 1 in endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	//08 25 01 00 00 00 00 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/
#endif

#if (CFG_I2S_IN_24BIT == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 2, Alternate 2 - alternate interface for data streaming */\
	//09 04 02 02 01 01 02 20 04
	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_MIC,           /* bInterfaceNumber */
	IN_ALT_SETTING_24BIT,                  /* bAlternateSetting */
	0x01,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	//10 24 01 13 00 01 01 00 00 00 01 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_MIC_OUTPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	0x01,                                  /* bmFormats UAC2_DATA_FORMAT_TYPE_I_PCM */
	0x00,
	0x00,
	0x00,
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	//06 24 02 01 04 18
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x03,                                 /* bSubSlotSizes */
	0x18,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	//07 05 81 05 84 01 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_IN_EP,                          /* bEndpointAddress 1 in endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	//08 25 01 00 00 00 00 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/
#endif

#if (CFG_I2S_IN_32BIT == 1)
	/* Standard AS Interface Descriptor(4.9.1) */
    /* Interface 2, Alternate 3 - alternate interface for data streaming */\
	//09 04 02 02 01 01 02 20 04
	UAC2_STD_AS_INTERFACE_DESC_SIZE,       /* bLength */
	USB_DESC_TYPE_INTERFACE,               /* bDescriptorType */
	ITF_NUM_AUDIO_STREAMING_MIC,           /* bInterfaceNumber */
	IN_ALT_SETTING_32BIT,                  /* bAlternateSetting */
	0x01,                                  /* bNumEndpoints */
	UAC2_INTERFACE_CLASS_AUDIO,            /* bInterfaceClass */
	UAC2_INTERFACE_SUBCLASS_STREAMING,     /* bInterfaceSubClass */
	UAC2_INTERFACE_PROTOCOL_CODE_V2,       /* bInterfaceProtocol */
	0x00,                                  /* iInterface */
	/* 09 byte */

	/* Class-Specific AS Interface Descriptor(4.9.2) */
	//10 24 01 13 00 01 01 00 00 00 01 00 00 00 00 00
	UAC2_CS_AS_INTERFACE_DESC_SIZE,        /* bLength */
	UAC2_CS_INTERFACE,                     /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_AS_GENERAL,       /* bDescriptorSubtype */
	UAC2_ENTITY_MIC_OUTPUT_TERMINAL,        /* bTerminalLink */
	0x00,                                  /* bmControls */
	UAC2_FORMAT_TYPE_I,                    /* bFormatType */
	0x01,                                  /* bmFormats UAC2_DATA_FORMAT_TYPE_I_PCM */
	0x00,
	0x00,
	0x00,
	0x02,                                  /* bNrChannels */
	U32_TO_U8S_LE(UAC2_CHANNEL_CONFIG_FRONT_LEFT | UAC2_CHANNEL_CONFIG_FRONT_RIGHT),  /* bmChannelConfig */
	0x00,                                  /* iChannelNames */
	/* 16 byte*/

	/* Type I Format Type Descriptor(2.3.1.6 - Audio Formats) */
	//06 24 02 01 04 18
	UAC2_FORMAT_TYPE_I_DESC_SIZE,         /* bLength */
	UAC2_CS_INTERFACE,                    /* bDescriptorType */
	UAC2_CS_AS_INTERFACE_FORMAT_TYPE,     /* bDescriptorSubtype */
	UAC2_FORMAT_TYPE_I,                   /* bFormatType */
	0x04,                                 /* bSubSlotSizes */
	0x20,                                 /* bBitResolution */
	/* 06 byte*/

	/* Standard AS Isochronous Audio Data Endpoint Descriptor(4.10.1.1) */
	//07 05 81 05 84 01 01
	UAC2_STANDARD_AS_ISO_EP_DESC_SIZE,    /* bLength */
	USB_DESC_TYPE_ENDPOINT,               /* bDescriptorType */
	AUDIO_IN_EP,                          /* bEndpointAddress 1 in endpoint*/
	USBD_EP_TYPE_ISOC_ASYNC,              /* bmAttributes */
	LOBYTE(MAX_PACKET_SIZE),              /* wMaxPacketSize in Bytes (192 + 1) * 2(Stereo) * 2(16bit)) = 772*/
	HIBYTE(MAX_PACKET_SIZE),
	0x01,                                 /* bInterval */
	/* 07 byte*/

	/* Class-Specific AS Isochronous Audio Data Endpoint Descriptor(4.10.1.2) */
	//08 25 01 00 00 00 00 00
	UAC2_CS_AS_ISO_EP_DESC_SIZE,          /* bLength */
	UAC2_CS_ENDPOINT,                     /* bDescriptorType */
	UAC2_CS_EP_SUBTYPE_GENERAL,           /* bDescriptorSubType */
	UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK, /* bmAttributes */
	UAC2_CTRL_NONE,                       /* bmControls */
	UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED, /* bLockDelayUnits */
	0x00,                                 /* wLockDelay */
	0x00,
	/* 08 byte*/
#endif
#endif
} ;
